import pandas as pd
import numpy as np
from upswingutil.db import Mongodb
from upswingutil.pms import PMS
from loguru import logger
import tensorflow as tf
from sklearn.model_selection import train_test_split
from sklearn.utils import resample
from numpy import zeros
from upswingutil.resource import get_model_from_cloud_storage, upload_model_to_cloud_storage


class Classification_Cancellation_NoShow_Others:
    def __init__(self, pms:str, orgId:str):
        self.pms = pms
        self.orgId = orgId

    def get_data(self) -> list:
        __pipeline__ = list()
        if self.pms == PMS.RMS:
            __pipeline__ = [
                {
                    '$addFields': {
                        'durationOfStay': {
                            '$floor': {
                                '$divide': [
                                    {
                                        '$subtract': [
                                            {
                                                '$toDate': '$departureDate'
                                            }, {
                                                '$toDate': '$arrivalDate'
                                            }
                                        ]
                                    }, 60 * 60 * 24 * 1000
                                ]
                            }
                        }, 
                        'bookingWindowDays': {
                            '$floor': {
                                '$divide': [
                                    {
                                        '$subtract': [
                                            {
                                                '$toDate': '$arrivalDate'
                                            }, {
                                                '$toDate': '$createdDate'
                                            }
                                        ]
                                    }, 60 * 60 * 24 * 1000
                                ]
                            }
                        }
                    }
                }, {
                    '$match': {
                        'guests': {
                            '$type': 4
                        }
                    }
                }, {
                    '$addFields': {
                        'didVisit': {
                            '$switch': {
                                'branches': [
                                    {
                                        'case': {
                                            '$eq': [
                                                '$status', 'Cancelled'
                                            ]
                                        }, 
                                        'then': 0
                                    }, {
                                        'case': {
                                            '$eq': [
                                                '$status', 'NoShow'
                                            ]
                                        }, 
                                        'then': 1
                                    }
                                ], 
                                'default': 2
                            }
                        }, 
                        'guestCount': {
                            '$size': '$guests'
                        }
                    }
                }, {
                    '$project': {
                        'didVisit': 1, 
                        'guestCount': 1, 
                        'durationOfStay': 1, 
                        'bookingWindowDays': 1, 
                        'Class': 1, 
                        'rateTypeName': 1, 
                        'clientId': 1, 
                        'rateTypeName': 1
                    }
                }
            ]

        elif self.pms == PMS.ORACLE:
            __pipeline__ = [
                {
                    '$match': {
                        'hotelId': 'SAND01', 
                        'guestInfo.adults': {
                            '$gte': 1
                        }
                    }
                }, {
                    '$project': {
                        'createBusinessDate': 1, 
                        'arrivalDate': 1, 
                        'departureDate': 1, 
                        'guestCount': '$guestInfo.adults', 
                        'rateTypeName': '$roomStay.ratePlanCode', 
                        'clientId': '$hotelId', 
                        'Class': '$roomStay.roomType', 
                        'status': 1
                    }
                }, {
                    '$match': {
                        'rateTypeName': {
                            '$type': 2
                        }
                    }
                }, {
                    '$addFields': {
                        'didVisit': {
                            '$switch': {
                                'branches': [
                                    {
                                        'case': {
                                            '$eq': [
                                                '$status', 'Cancelled'
                                            ]
                                        }, 
                                        'then': 0
                                    }, {
                                        'case': {
                                            '$eq': [
                                                '$status', 'NoShow'
                                            ]
                                        }, 
                                        'then': 1
                                    }
                                ], 
                                'default': 2
                            }
                        }, 
                        'durationOfStay': {
                            '$floor': {
                                '$divide': [
                                    {
                                        '$subtract': [
                                            {
                                                '$toDate': '$departureDate'
                                            }, {
                                                '$toDate': '$arrivalDate'
                                            }
                                        ]
                                    }, 60 * 60 * 24 * 1000
                                ]
                            }
                        }, 
                        'bookingWindowDays': {
                            '$floor': {
                                '$divide': [
                                    {
                                        '$subtract': [
                                            {
                                                '$toDate': '$arrivalDate'
                                            }, {
                                                '$toDate': '$createBusinessDate'
                                            }
                                        ]
                                    }, 60 * 60 * 24 * 1000
                                ]
                            }
                        }
                    }
                }, {
                    '$project': {
                        'arrivalDate': 0, 
                        'createBusinessDate': 0, 
                        'departureDate': 0, 
                        'status': 0
                    }
                }
            ]

        mongo = Mongodb(self.orgId)
        result = mongo.execute_pipeline(mongo.RESERVATION_COLLECTION, __pipeline__)
        mongo.close_connection()
        return result
        
    def preprocess(self):
        data = pd.DataFrame(self.get_data())
        logger.info(f'DataFrame Created from the Data Pipeline !!')
        data = data.query('guestCount > 0')
        features = data.iloc[:, 1:]
        if self.pms == PMS.RMS:
            features['rateTypeName'] = features['rateTypeName'].replace("", 'None')
        labels = features['didVisit']
        features = features.drop('didVisit', axis=1)

        ##Feature Engineering (one Hot Encoding the categories)
        #1. THis approach below is very easy but can not be replicated when predictions, we need a trained model to perform the same one hot encoding 
        features = pd.get_dummies(features, columns=['clientId', 'rateTypeName', 'Class'])

        ##2. Therefore, we are using the OneHot Encoding method to train a scikit learn model and furhter be able to use it during prediction
        # self.one_hot = OneHotEncoder(sparse=False, drop='first')
        # self.one_hot.fit(features[['clientId','rateTypeName', 'Class']])
        # upload_model_to_cloud_storage(self.one_hot, self.orgId, 'Classification_Cancellation_NoShow_Others__ONE-HOT-ENCODING.pkl')
        # join = pd.DataFrame(self.one_hot.transform(features[['clientId','rateTypeName', 'Class']]))
        # features = pd.concat([features, join],axis=1).drop(['rateTypeName', 'Class', 'clientId'], axis=1)

        X_train, self.X_test, y_train, self.y_test = train_test_split(features, labels, test_size=0.3, random_state=42)
        logger.info(f'Train-Test Split Successful !!')
        ## Upsampling to deal with Skewed Distributed Class
        train_data = pd.concat([X_train, y_train], axis=1)
        negative = train_data[train_data.didVisit==0]
        positive = train_data[train_data.didVisit==2]
        neutral = train_data[train_data.didVisit==1]

        # upsample minority
        neg_upsampled = resample(negative,
        replace=True, # sample with replacement
        n_samples=len(positive), # match number in majority class
        random_state=27) # reproducible results

        neu_upsampled = resample(neutral,
        replace=True, # sample with replacement
        n_samples=len(positive), # match number in majority class
        random_state=27)
        # combine majority and upsampled minority
        upsampled = pd.concat([positive, neg_upsampled, neu_upsampled])
        upsampled = upsampled.sample(frac=1)
        self.labels = upsampled.didVisit
        self.features = upsampled.drop('didVisit', axis=1)
        logger.info(f'Upsampled Training Features and Labels are Created !!')


    def train(self):

        try:
            ## MODEL ARCHITECTURE CREATION
            self.model = tf.keras.Sequential([
                tf.keras.layers.Dense(64, activation='relu', input_shape=(self.features.shape[1],)),
                tf.keras.layers.Dense(32, activation='relu',activity_regularizer=tf.keras.regularizers.L2(0.01)),
                tf.keras.layers.Dense(32, activation='relu',activity_regularizer=tf.keras.regularizers.L2(0.01)),
                tf.keras.layers.Dense(3, activation='softmax')
            ])

            logger.info("Model is Successfully Initialized...")
            
            ## COMPILATION PHASE
            self.model.compile(loss='sparse_categorical_crossentropy', optimizer='adam', metrics=['accuracy'])

            ## TRAINING PHASE
            self.history = self.model.fit(self.features, self.labels, batch_size=32, epochs=8, validation_data=(self.X_test, self.y_test), verbose=0)
            logger.info('Model is Successfully Trained, Here is the Metrics of the Model training:-->')
            logger.info(f"\tLoss: {self.history.history['loss'][-1] :.4f}\n \
                \tValidation Loss: {self.history.history['val_loss'][-1] :.4f} \
                \tAccuracy: {self.history.history['accuracy'][-1]*100 :.4f} \
                \tValidation Accuracy: {self.history.history['val_accuracy'][-1]*100 :.4f}  \
                ")

            try:
                upload_model_to_cloud_storage(self.model, self.orgId, 'Classification_Cancellation_NoShow_Others.pkl')
                logger.info("upload_model_to_cloud_storage Successfully ...")
            except Exception as e:
                self.model.save('Classification_Cancellation_NoShow_Others')
                logger.info("Tensorflow Model Saved!! --> model.save() ")


        except Exception as e:
            logger.info(f"Model is not Created, Some Error: {e}, with Model Initlization/Compilation/Training !")

    def predict(self, clientId:int, rateTypeName:str, Class:str, durationOfStay:float, bookingWindowDays:float, guestCount:int):

        try:
            self.model = get_model_from_cloud_storage(self.orgId, "Classification_Cancellation_NoShow_Others.pkl")
            logger.info("Loading of the Model from the Cloud is Successful!")
        except Exception as e:
            logger.info(f'Error in Loading: {e}')
            self.model = tf.saved_model.load('Classification_Cancellation_NoShow_Others')

        if self.pms == PMS.RMS:
            test_shape = 34
            columns = ['durationOfStay', 'bookingWindowDays', 'guestCount', 'clientId_0',
        'clientId_11263', 'clientId_11264', 'clientId_11286', 'clientId_11288',
        'clientId_11289', 'clientId_11290', 'clientId_11347', 'clientId_12053',
        'clientId_12333', 'clientId_12952', 'clientId_13773', 'clientId_14170',
        'clientId_14277', 'clientId_14278', 'rateTypeName_',
        'rateTypeName_BAR (Multi)', 'rateTypeName_BAR Rate',
        'rateTypeName_BAR_Airbnb', 'rateTypeName_BAR_BB',
        'rateTypeName_BAR_Hotelbeds',
        'rateTypeName_BAR_Hotelbeds_Non Refundable',
        'rateTypeName_BAR_Non Refundable',
        'rateTypeName_Direct Booking_Internal Rate',
        'rateTypeName_Old system rate', 'rateTypeName_Owner House Use ONLY',
        'Class_Good', 'Class_Loyal', 'Class_None', 'Class_Temporary',
        'Class_VIP']
        elif self.pms == PMS.ORACLE:
            test_shape = 17
            columns = None

        test = pd.DataFrame(zeros(shape=(1,test_shape), dtype=np.int), columns=columns)

        test['durationOfStay'] = durationOfStay
        test['bookingWindowDays'] = bookingWindowDays
        test['guestCount'] = guestCount
        test[f'Class_{Class}'] = 1
        test[f'rateTypeName_{rateTypeName}'] = 1
        test[f'clientId_{clientId}'] = 1

        pred = self.model.predict(test)
        final = pred.argmax()
        d = {
                0:'Cancelled',
                1:'No Show',
                2:'Others'
            }
        final = d[final]
        overall = {d[i]:round(pred[0][i]*100, 4) for i in range(3)}
        return final, overall


        



import unittest
import firebase_admin
from firebase_admin import credentials

import upswingutil as ul
from upswingutil.pms.rms import ReservationSync
from upswingutil.resource import access_secret_version, setup_logging
import json


class TestRMSReservationV2API(unittest.TestCase):
    ul.G_CLOUD_PROJECT = 'aura-staging-31cae'
    ul.ENCRYPTION_SECRET = "S1335HwpKYqEk9CM0I2hFX3oXa5T2oU86OXgMSW4s6U="
    ul.MONGO_URI = access_secret_version(ul.G_CLOUD_PROJECT, 'MONGOURI', '1')

    def test_adding_new_reservation(self):
        setup_logging()
        print('Initializing default firebase app')
        cred = json.loads(
            access_secret_version(ul.G_CLOUD_PROJECT, 'Firebase-Aura', '1'))
        cred = credentials.Certificate(cred)
        firebase_admin.initialize_app(cred)

        print('Initializing alvie firebase app')
        cred = json.loads(
            access_secret_version(ul.G_CLOUD_PROJECT, 'Firebase-Alvie', '1'))
        cred = credentials.Certificate(cred)
        firebase_admin.initialize_app(cred, name='alvie')

        resv = ReservationSync('11249')
        resv_list = [1]
        for _resv_id in resv_list:
            print(f'processing reservation : {_resv_id}')
            resv.process(_resv_id)
            print(f'processed reservation : {_resv_id}')



import pandas as pd
import numpy as np
from statsmodels.tsa.holtwinters import SimpleExpSmoothing
from statsmodels.tsa.arima.model import ARIMA
from upswingutil.db import Mongodb
from datetime import datetime
from upswingutil.db.model import Status
from loguru import logger
from upswingutil.resource import upload_model_to_cloud_storage, get_model_from_cloud_storage
from upswingutil.pms.rms import NAME as RMS
from upswingutil.pms.oracle import NAME as ORACLE


class ReservationCountForecast:
    """
    Predict reservation count for a given date range.
    """
    __file_name__ = 'booking_count_forecast_{}.pkl'

    def __init__(self, pms, orgId, propertyId: str):
        self.pms = pms
        self.orgId = orgId
        self.propertyId = propertyId

    def get_data(self, start_date, end_date):
        mongo = Mongodb(self.orgId)
        __pipeline__ = list()
        if self.pms == RMS:
            __pipeline__ = [
                {
                    '$match': {
                        "clientId": int(self.propertyId),
                        'status': {
                            '$nin': [
                                Status.MAINTENANCE.value
                            ]
                        }
                    }
                }, {
                    '$project': {
                        'Time': '$arrivalDate'
                    }
                }, {
                    '$addFields': {
                        'Time': {
                            '$substr': [
                                '$Time', 0, 10
                            ]
                        }
                    }
                }, {
                    '$addFields': {
                        'Time': {
                            '$dateFromString': {
                                'dateString': '$Time'
                            }
                        }
                    }
                }, {
                    '$group': {
                        '_id': '$Time',
                        'Counts': {
                            '$sum': 1
                        }
                    }
                }, {
                    '$sort': {
                        '_id': 1
                    }
                }, {
                    '$match': {
                        '_id': {
                            '$gte': start_date
                        }
                    }
                }, {
                    '$match': {
                        '_id': {
                            '$lte': end_date
                        }
                    }
                }
            ]
        elif self.pms == ORACLE:
            __pipeline__ = [
                {
                    '$match': {
                        'hotelId': self.propertyId,
                        'status': {
                            '$nin': [
                                Status.MAINTENANCE.value
                            ]
                        },
                        'arrivalDate': {
                            '$gte': start_date,
                            '$lte': end_date
                        }
                    }
                }, {
                    '$project': {
                        'Time': '$arrivalDate'
                    }
                }, {
                    '$addFields': {
                        'Time': {
                            '$substr': [
                                '$Time', 0, 10
                            ]
                        }
                    }
                }, {
                    '$addFields': {
                        'Time': {
                            '$dateFromString': {
                                'dateString': '$Time'
                            }
                        }
                    }
                }, {
                    '$group': {
                        '_id': '$Time',
                        'Counts': {
                            '$sum': 1
                        }
                    }
                }, {
                    '$sort': {
                        '_id': 1
                    }
                }
            ]
        result = mongo.execute_pipeline(mongo.RESERVATION_COLLECTION, __pipeline__)
        mongo.close_connection()
        return result

    def train(self):
        logger.info(f'Training reservation count forecasting for property {self.propertyId}')
        file_name = ReservationCountForecast.__file_name__.format(self.propertyId)
        start_date = pd.Timestamp("2018-01-01")
        end_date = pd.Timestamp(datetime.now().strftime('%Y-%m-%d'))
        df = pd.DataFrame(self.get_data(start_date, end_date))
        if df.shape[0] == 0:
            print(f"Property:: {self.propertyId} Doesn't Have any Data")
            return
        df["_id"] = pd.DatetimeIndex(df["_id"])
        df = pd.DataFrame(pd.date_range(start_date, end_date), columns=["_id"]).merge(df, on=["_id"],
                                                                                      how="outer").fillna(0)
        df.set_index("_id", inplace=True)
        if df.shape[0] > 10:
            smooth_model = SimpleExpSmoothing(np.asarray(df.Counts), initialization_method='heuristic')
            smooth_model_fitted = smooth_model.fit(smoothing_level=0.3)
            yhat = smooth_model_fitted.fittedvalues[:]
        else:
            yhat = df.Counts

        arima_model = ARIMA(yhat, order=(3, 1, 2))
        fitted_model = arima_model.fit()
        upload_model_to_cloud_storage(fitted_model, self.orgId, file_name)
        logger.info(f"New Model created, File saved: {file_name}")
        logger.info(f"Completed Reservation count forecasting model for {self.propertyId}")

    def predict(self, start_date, end_date):
        file_name = ReservationCountForecast.__file_name__.format(self.propertyId)
        fitted_model = get_model_from_cloud_storage(self.orgId, file_name)

        start_ = pd.Timestamp("2018-01-01")
        p_start = pd.Timestamp(start_date)
        pred_time = pd.date_range(start_date, end_date, normalize=True)
        start_index = (p_start - start_).days  # df[df._id == pd.to_datetime(start)].index.values[0] ## I used dataframe but you have to mention it from your side, Like revenue forecasting
        end_index = start_index + pred_time.shape[0]
        pred_values = fitted_model.get_prediction(start=start_index, end=end_index).predicted_mean

        return pred_time, pred_values[1:]



from datetime import datetime
from upswingutil.resource import get_model_from_cloud_storage
from upswingutil.pms.rms import NAME, get_key, validate_key
from upswingutil.db import Mongodb, Firestore
from upswingutil.schema import Token
import upswingutil as ul
import requests
import logging


def _get_geo_region(nationalID: int):
    result = 'unknown'
    try:
        mongo = Mongodb('upswing')
        region = mongo.get_collection('countries').find_one({"_id": nationalID}, {"region": 1})
        mongo.close_connection()
        result = region if region else 'unknown'
    except Exception as e:
        logging.error(f'Error getting geo region of nation {nationalID}')
        logging.error(e)
    finally:
        return result


@DeprecationWarning
class ReservationSync:

    def __init__(self, orgId: str, g_cloud_token=None):
        self.orgId = orgId
        self.mongo = Mongodb(orgId)
        self.token: Token = get_key(self.orgId)
        self.g_cloud_token = g_cloud_token
        self.__urgent_booking_criteria__ = 5 * 86400
        self.__stay_duration_criteria__ = 10 * 86400
        self.__booking_category_levels__ = ['bronze', 'silver', 'gold', 'platinum']

    def _get_booking_type(self, arrival_date: str, created_date: str):
        result = 'unknown'
        try:
            delta = datetime.fromisoformat(arrival_date) - datetime.fromisoformat(created_date)
            delta = delta.days * 86400 + delta.seconds
            result = 'Urgent Booking' if delta < self.__urgent_booking_criteria__ else 'Pre Planned'
        except Exception as e:
            logging.error('Error while calculating booking type')
            logging.error(e)
        finally:
            return result

    def _get_duration_type(self, departure_date: str, arrival_date: str):
        result = 'unknown'
        try:
            stay_d = datetime.fromisoformat(departure_date) - datetime.fromisoformat(arrival_date)
            delta = stay_d.days * 86400 + stay_d.seconds
            result = 'Short Stay' if delta < self.__stay_duration_criteria__ else 'Long Stay'
        except Exception as e:
            logging.error('Error calculating stay duration type')
            logging.error(e)
        finally:
            return result

    def _get_booking_level(self, orgId, clientId, departure_date: str, arrival_date: str, spend, booking_type):
        result = 'unknown'
        try:
            stay_d = datetime.fromisoformat(departure_date) - datetime.fromisoformat(arrival_date)
            booking_type_id = 1 if booking_type == 'Urgent Booking' else 0
            model = get_model_from_cloud_storage(
                str(orgId),
                f'booking_categorization_{clientId}.pkl',
                token=self.g_cloud_token
            )
            prediction = model.predict([[stay_d.days, spend, booking_type_id]])
            level = prediction[0] if len(prediction) > 0 else None
            result = self.__booking_category_levels__[level] if level else 'unknown'
        except Exception as e:
            logging.error('Error calculating booking_level of reservation')
            logging.error(e)
        finally:
            return result

    def _retrieve_data(self, id, name, url):
        result = None
        try:
            logging.debug(f'Extracting {name} for reservation: {id}')
            header = {
                'Content-Type': 'application/json',
                'authtoken': self.token.key
            }
            response = requests.request("GET", url, headers=header)
            if response.status_code == 200:
                result = response.json()
            else:
                logging.error(f'Error getting {name} for reservation {id} due to status code {response.status_code}')
                logging.error(url.format(id))
        except Exception as e:
            logging.error(f' {name} failed due to {e}')
        finally:
            return result

    def extract_reservation_details(self, record):
        feature_list = [
            {
                'name': 'holds',
                'url': 'holds'
            },
            {
                'name': 'guests',
                'url': 'guests'
            },
            {
                'name': 'billTo',
                'url': 'billTo'
            },
            {
                'name': 'transfers',
                'url': 'transfers'
            },
            {
                'name': 'auditTrail',
                'url': 'auditTrail'
            },
            {
                'name': 'daily_rates',
                'url': 'dailyRates'
            },
            {
                'name': 'rego_access',
                'url': 'regoAccess'
            },
            {
                'name': 'requirement',
                'url': 'requirements'
            },
            {
                'name': 'housekeeping',
                'url': 'housekeeping'
            },
            {
                'name': 'daily_revenue',
                'url': 'dailyRevenue'
            },
            {
                'name': 'add_ons',
                'url': 'reservationAddOn'
            },
            {
                'name': 'correspondence',
                'url': 'correspondence'
            },
            {
                'name': 'financial_info_actual',
                'url': 'actualAccount'
            },
            {
                'name': 'bedConfiguration',
                'url': 'bedConfiguration'
            },
        ]
        try:
            logging.info(f"Extracting reservation {record.get('id')} additional details.")

            for item in feature_list:
                _url = f"{self.token.hostName}/reservations/{record.get('id')}/{item.get('url')}"
                record[item.get('name')] = self._retrieve_data(record.get("id"), item.get("name"), _url)
            return record
        except Exception as e:
            logging.error(f'Error retrieving addition reservation details for {record.get("id")}')
            logging.error(e)

    def transform_reservation(self, record):
        try:
            logging.info(f'Transforming reservation {record.get("id")}')
            record['booking_type'] = self._get_booking_type(record['arrivalDate'], record['createdDate'])
            record['duration_type'] = self._get_duration_type(record['departureDate'], record['arrivalDate'])
            record['globalId'] = f"{record.get('agent')}-{record.get('orgId')}-{record.get('clientId')}-{record.get('id')}"
            record['booking_level'] = self._get_booking_level(
                record.get('orgId'),
                record['clientId'], record['departureDate'],
                record['arrivalDate'], record['financial_info_actual']['totalRate'],
                record['booking_type']
            )
            return record
        except Exception as e:
            logging.error(f"Exception while transforming reservation {record.get('id')}")
            logging.error(e)

    def transform_guest(self, record):
        try:
            logging.info(f'Transforming guests of reservation {record.get("id")}')
            for guest in record.get('guests'):
                guest['_id'] = guest.get('id')
                del guest['id']
                guest['geoRegion'] = _get_geo_region(guest.get('nationalityId'))
            return record
        except Exception as e:
            logging.error(f'Error while transforming guest for reservation {record.get("id")}')
            logging.error(e)

    def _add_reservation_to_alvie(self, record):
        firestore_db = Firestore(app='alvie')
        user_email_list = [item["email"] for item in record.get('guests')]
        resv_info = {
            "_id": record.get("id"),
            "agent": record.get("agent"),
            "orgId": record.get("orgId"),
            "areaId": record.get("areaId"),
            "arrivalDate": record.get("arrivalDate"),
            "departureDate": record.get("departureDate"),
            "propertyId": record.get("propertyId"),
            "propertyName": record.get("propertyName"),
            "status": record.get("status"),
            "travelAgentId": record.get("travelAgentId"),
            "travelAgentName": record.get("travelAgentName")
        }
        logging.debug(f"Final reservation id : {record.get('id')}")
        for email in user_email_list:
            docs = firestore_db.get_collection('users').where("email", "==", email).stream()
            for doc in docs:
                logging.info("loading reservation to alvie")
                firestore_db.get_collection(f'users/{doc.id}/reservations') \
                    .document(str(record.get("id"))) \
                    .set(resv_info, merge=True)
                logging.debug(f"added to {doc.id}")

    def extract_reservation(self, record):
        reservationId = record.get("reservation")
        logging.info(f'Extracting reservation: {reservationId}')

        header = {
            'Content-Type': 'application/json',
            'authtoken': self.token.key
        }
        url = f"{self.token.hostName}/reservations/{reservationId}?modelType=full"
        response = requests.request("GET", url, headers=header)
        if response.status_code == 200:
            try:
                response_json = dict(response.json())
                response_json['orgId'] = self.orgId
                response_json['agent'] = record.get('agent')
                property_info = self.mongo.get_collection(self.mongo.PROPERTY_COLLECTION).find_one(
                    {"areas.id": response_json.get('areaId')},
                    {'id': 1, 'name': 1,
                     'clientId': 1}
                )
                response_json['clientId'] = property_info.get('clientId') if property_info else 0
                response_json['propertyId'] = property_info.get('_id') if property_info else 0
                response_json['propertyName'] = property_info.get('name') if property_info else 'Unknown'
                return response_json
            except Exception as err:
                logging.error(f'{record} load failed due to {err}')
        else:
            logging.error(f"RMS returned status code {response.status_code} "
                          f"for reservation {reservationId} ")
            raise Exception('Reservation not found / unable to retrieve')

    def _load_to_mongodb(self, record):
        _id = record.get("id")
        orgId = record.get("orgId")
        record.update({'lastUpdate': str(datetime.now())})
        try:
            logging.info(f'Adding reservation {_id} to db {orgId}')
            self.mongo.get_collection(self.mongo.RESERVATION_COLLECTION).update_one({'_id': _id},
                                                                          {'$set': record}, upsert=True)
        except Exception as e:
            logging.error(f'Error storing to respective {_id} to db {orgId}')
            logging.error(e)

    def process(self, reservationId):
        record = {
            'orgId': self.orgId,
            'agent': NAME,
            'reservation': reservationId
        }

        if not validate_key(self.token.validity):
            logging.info('Refreshing RMS token as it is about to expire')
            self.token = get_key(self.orgId)

        record = self.extract_reservation(record)
        record = self.extract_reservation_details(record)
        record = self.transform_reservation(record)
        record = self.transform_guest(record)
        self._load_to_mongodb(record)
        self._add_reservation_to_alvie(record)

    def __del__(self):
        if self.mongo:
            self.mongo.close_connection()


if __name__ == '__main__':
    ul.G_CLOUD_PROJECT = 'aura-staging-31cae'
    ul.ENCRYPTION_SECRET = "S1335HwpKYqEk9CM0I2hFX3oXa5T2oU86OXgMSW4s6U="
    ul.FIREBASE = '/Users/harsh/upswing/github/agent-oracle/SECRET/aura-staging-31cae-firebase-adminsdk-dyolr-' \
                  '7c135838e9.json'
    ul.MONGO_URI = "mongodb://AdminUpSwingGlobal:Upswing098812Admin0165r@dev.db.upswing.global:27017/?authSo" \
                   "urce=admin&readPreference=primary&appname=Agent%20RMS%20Dev&ssl=false"
    import firebase_admin
    cred = firebase_admin.credentials.Certificate(ul.FIREBASE)
    firebase = firebase_admin.initialize_app(cred)
    resv = ReservationSync('11249')
    resv.process(23560)
    resv.process(23590)



import urllib
import requests
from datetime import datetime, timedelta
from upswingutil.db import Firestore
from upswingutil.resource import decrypt
from upswingutil.schema import Token
from loguru import logger
from amadeus import Client


def validate_key(expiryDate):
    """
    Validated whether the token is still valid or not
    :param expiryDate:
    :return:
    """
    try:
        if expiryDate is None:
            return False
        return datetime.fromisoformat(expiryDate) > (datetime.now() + timedelta(minutes=5))
    except Exception as e:
        logger.error('Error while validating key')
        logger.error(e)
        return False


def get_key(production: bool = True) -> Client:
    """
    This method generates auth token for provided org_id, in case of refreshing token,
    provide refresh_token, to generate new token using it.

    :return: instance of Token
    """
    try:
        db = Firestore()
        production = 'production' if production else 'test'
        secret = db.get_collection(db.credentials).document('amadeus').get().to_dict()
        amadeus = Client(
            hostname=production,
            client_id=secret.get('clientId'),
            client_secret=secret.get('clientSecret')
        )
        logger.info(f'Amadeus token generated for : {amadeus.hostname} env')
        return amadeus
    except Exception as e:
        logger.error('Error generating token key oracle')
        logger.error(e)


if __name__ == '__main__':
    import upswingutil as ul
    import os
    import json
    import firebase_admin
    ul.G_CLOUD_PROJECT = 'aura-staging-31cae'
    cred = json.loads(ul.resource.access_secret_version(os.getenv('G_CLOUD_PROJECT', 'aura-staging-31cae'), 'Firebase-Aura', '1'))
    cred = firebase_admin.credentials.Certificate(cred)
    firebase = firebase_admin.initialize_app(cred)
    amadeus = get_key(production=False)



# from datetime import timedelta
# from upswingutil.db import Mongodb
# from upswingutil.resource import get_model_from_cloud_storage
# import pandas as pd
#
#
# __file_path__ = 'app/resource/ml/models/revenue_forecast'
# __df_pickle_file__ = 'app/resource/ml/revenue_forecast_df_{}.pkl'
# months = ['', 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
#
#
# class RevenueForecast:
#
#     def __init__(self, propId, orgId):
#         self.propId = int(propId)
#         self.orgId = str(orgId)
#         self.db = None
#
#     def forecast(self, start, end):
#         self.db = Mongodb(self.orgId)
#         fitted_model = get_model_from_cloud_storage('revenue_forecast', f'revenue_forecast_{self.propId}.pkl')
#         df = get_model_from_cloud_storage('revenue_forecast', f'revenue_forecast_df_{self.propId}.pkl')
#
#         actual_time = df.loc[start:].index
#         today = pd.to_datetime('today')
#         end_2 = pd.date_range(today, end, normalize=True)
#         pred_time = actual_time.append(end_2[1:])
#         df2 = df.reset_index()
#         start_index = df2[df2["_id"] == start].index.values[0]
#         end_index = start_index + pred_time.shape[0]
#         pred_values = fitted_model.get_prediction(start=start_index, end=end_index).predicted_mean
#
#         forecast_array = list()
#         for date, forecast in zip(pred_time, pred_values[1:]):
#             if date < end:
#                 pred = {
#                     "x": str(date.day) + " " + date.month_name()[:3] + " " + str(date.year),
#                     "y": forecast
#                 }
#                 forecast_array.append(pred)
#
#         return forecast_array
#
#     def __del__(self):
#         if self.db:
#             self.db.close_connection()
#
#
# def transform_date(date):
#     return f'{date.day} {months[date.month]} {date.year}'
#
#
# def revenue_forecast_for_property_by_steps(clientId, time_steps):
#     fitted_model = get_model_from_cloud_storage('revenue_forecast', f'revenue_forecast_{clientId}.pkl')
#     forecast_array = fitted_model.forecast(int(time_steps))
#     return list(forecast_array)
#
#
# def revenue_predict_for_property_by_range(clientId, from_date, to_date):
#     days = (to_date - from_date).days
#     result = list()
#     _arr = revenue_forecast_for_property_by_steps(clientId, days)
#     print(days)
#     print(len(_arr))
#     for i in range(days):
#         date = from_date + timedelta(days=i)
#         result.append({
#             "x": str(date.strftime('%d %b %Y')),
#             "y": _arr[i]
#         })
#
#     return result
